from enum import StrEnum
from typing import Literal, Optional, Union

from pydantic import BaseModel, ConfigDict, Field


class AutosuggestedSearch(BaseModel):
    """Class used only to parse the output from the Autosuggestion"""

    # TODO: Move this to bigdata.api.autosuggest?

    model_config = ConfigDict(populate_by_name=True)
    id: Optional[str] = Field(validation_alias="key", default=None)
    name: Optional[str] = Field(default=None)
    query_type: Literal["savedSearch"] = Field(
        default="savedSearch", validation_alias="queryType"
    )


class ExpressionOperation(StrEnum):
    IN = "in"


class ExpressionTypes(StrEnum):
    AND = "and"
    KEYWORD = "keyword"
    ENTITY = "entity"
    SOURCE = "source"
    TOPIC = "rp_topic"
    LANGUAGE = "language"
    WATCHLIST = "watchlist"
    DATE = "date"


class Expression(BaseModel):
    type: ExpressionTypes
    value: Union[list[Union[str, "Expression"]], str]
    operation: Optional[ExpressionOperation] = None

    @classmethod
    def new(cls, etype: ExpressionTypes, values: Optional[list[str]]) -> "Expression":
        if not values:
            return None
        return cls(type=etype, operation=ExpressionOperation.IN, value=values)


class FileType(StrEnum):
    ALL = "all"
    FILINGS = "filings"
    TRANSCRIPTS = "transcripts"
    NEWS = "news"
    FILES = "files"


class SortBy(StrEnum):
    """Defines the order of the search results"""

    RELEVANCE = "relevance"
    DATE = "date"


class Ranking(StrEnum):
    STABLE = "stable"
    EXPERIMENTAL = "experimental"
    SIMILARITY = "similarity"


class SearchChain(StrEnum):
    DEDUPLICATION = "deduplication"
    ENRICHER = "enricher"  # NO LONGER USED
    DEFAULT = "default"  # NO LONGER USED?
    CLUSTERING = "clustering"


class SearchPagination(BaseModel):
    limit: int = Field(default=100, gt=0, lt=1001)
    cursor: int = Field(default=1, gt=0)
